#include <jni.h>
#include "iterate_IterateJNI.h"
#include <stdio.h>

/* This is a simple Hellow world like thing to test communication Java <-> C */
JNIEXPORT void JNICALL Java_iterate_IterateJNI_helloWorld(JNIEnv *env, jobject obj)
{
	printf("Hello world from C!\n");
	return;
}


/* This tests the handling of doubles. */
JNIEXPORT void JNICALL Java_iterate_IterateJNI_helloDouble(JNIEnv *env, jobject obj, jdouble d)
{
	printf("our double is:%lf\n" ,d );
	return;
}


/* This tests the handling of Arrays. */
JNIEXPORT void JNICALL Java_iterate_IterateJNI_helloArray(JNIEnv *env, jobject obj, jintArray arr)
{
	
  jsize len = (*env)->GetArrayLength(env, arr);
  int i, sum = 0;
  jint *body = (*env)->GetIntArrayElements(env, arr, 0);
  
  for (i=0; i<len; i++) {
    body[i] +=3;
  }

  (*env)->ReleaseIntArrayElements(env, arr, body, 0);
  return;
}


/* This is the JNI implementation of the mandelbrot calculation. */
JNIEXPORT jint JNICALL Java_iterate_IterateJNI_iterate(JNIEnv *env, jobject obj,
		jdouble x1,
		jdouble x2,
		jdouble y1,
		jdouble y2,
		jint iter,
		jint w,
		jint h,
		jintArray arr)
{
	double xd,yd,x,y,xs,ys,e,f;
	int count, px, py,n;
	jint *body;
	
  	jsize len = (*env)->GetArrayLength(env, arr);

    xd= (x2 - x1) / (w-1);
    yd= (y2 - y1) / (h-1);

	n= 0;
	
	// check len
	if (len != w*h)
		return 1;

	// alloc(!)
  	body = (*env)->GetIntArrayElements(env, arr, 0);
	
	xs= x1;
	for (px= 0; px < w; px++)
    {
        ys= y1;
        for (py=0; py<h; py++)   
        {
            count= 0;
			x= xs;
			y= ys;
			do
			{
				e= x * x;
				f= y * y;
				y= 2.0 * x * y + ys;
				x= e - f + xs;
			   count++;
			}
			while (count < iter && ((e + f) < 4.0));
			body[n++]= count;
			ys += yd;
		}
		xs += xd;
	}
	// dont forget the free(!)
  	(*env)->ReleaseIntArrayElements(env, arr, body, 0);
	return 0;
}


