
package mandel;
import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Event;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Rectangle;
import java.awt.SystemColor;
import java.awt.TextField;
import java.util.Iterator;

public class MandelMain extends Applet
{
	private double xmin, xmax, ymin, ymax;

	private PresetManager presets= new PresetManager();

	private Panel p= new Panel();
	private TextField tf_xmin; // X and Y max and min input fields
	private TextField tf_xmax;
	private TextField tf_ymin;
	private TextField tf_ymax; // Drop-down box with predefined views
	private Choice Pal_box= new Choice(); // Drop-down box with palette choices
	private Choice Predef_box= new Choice();

	private Button Render_btn= new Button(); // Button to execute
	private MandelCanvas mandelCanvas1= new MandelCanvas(this); // Special Canvas class
	private boolean rp_b;
	private Label label1= new Label();
	private Label label2= new Label();
	private Label label3= new Label();
	private Label label4= new Label();
	private Label label5= new Label();
	private Label label6= new Label();
	private TextField tfiterations= new TextField();
	private Label label7= new Label(); // True if RainbowPalette, false if

	public void init()
	{
		presets.init();
		setLayout(new BorderLayout());
		Predef_box= new Choice();

		for (Iterator iterator= presets.iterator(); iterator.hasNext();)
		{
			Preset pr= (Preset) iterator.next();
			Predef_box.addItem(pr.name);
		}

		Pal_box= new Choice();
		Pal_box.addItem("Rainbow");
		Pal_box.addItem("Gray Scale");
		rp_b= true;
		try
		{
			jbInit();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	//Component initialization
	private void jbInit() throws Exception
	{

		tf_xmin= new TextField("-2.25");
		tf_xmin.setBounds(new Rectangle(10, 182, 183, 21));

		tf_xmax= new TextField("0.75");
		tf_xmax.setBounds(new Rectangle(10, 141, 183, 21));
		tfiterations.setText("500");
		tfiterations.setBounds(new Rectangle(9, 262, 183, 21));
		label7.setBounds(new Rectangle(9, 241, 183, 19));
		label7.setText("Iterations");
		p.add(Pal_box);

		tf_ymin= new TextField("-1.5");
		tf_ymin.setBounds(new Rectangle(10, 96, 183, 21));

		tf_ymax= new TextField("1.5");

		p.add(label6, null);
		p.add(Predef_box);
		p.add(label4, null);
		p.add(tf_ymin);
		p.add(label3, null);
		p.add(label2, null);
		p.add(tf_xmax);
		p.add(tf_xmin);
		p.add(label1, null);
		p.add(tf_ymax);
		p.add(tfiterations, null);
		p.add(label7, null);
		p.add(label5, null);
		p.add(Pal_box);
		p.add(Render_btn);
		tfiterations= new TextField("1.5");
		mandelCanvas1= new MandelCanvas(this);
		this.setBackground(Color.gray);
		this.setLayout(null);

		p.setLayout(null);
		p.setBackground(SystemColor.control);
		p.setSize(200, 200);
		p.setBounds(new Rectangle(468, 2, 210, 439));
		Predef_box.setBounds(new Rectangle(9, 30, 183, 22));
		Render_btn.setLabel("Preview");
		Render_btn.setBounds(new Rectangle(12, 366, 98, 21));

		tf_ymax.setBounds(new Rectangle(9, 217, 183, 21));
		Pal_box.setBounds(new Rectangle(10, 329, 183, 22));
		label1.setText("X-Min");
		label1.setBounds(new Rectangle(10, 202, 183, 19));
		label2.setBounds(new Rectangle(10, 164, 183, 19));
		label2.setText("X-Max");
		label3.setText("y-Min");
		label3.setBounds(new Rectangle(10, 119, 183, 19));
		label4.setText("Y-Max");
		label4.setBounds(new Rectangle(10, 74, 183, 19));
		label5.setText("Palette");
		label5.setBounds(new Rectangle(11, 306, 183, 19));
		label6.setText("Preset");
		label6.setBounds(new Rectangle(11, 10, 183, 19));

		p.add(Render_btn);
		mandelCanvas1.setBounds(new Rectangle(1, 1, 300, 300));
		this.add(mandelCanvas1, null);
		this.add(p, null);
	}

	// handleEvent(), in this case, only destroys the applet if
	// the parent window (browser) is closed.
	public boolean handleEvent(Event evt)
	{
		if (evt.id == Event.WINDOW_DESTROY)
			System.exit(0);
		return super.handleEvent(evt);
	}

	// action() handles all user events, such as drop-down box
	// selects, text input, and button clicks.
	public boolean action(Event evt, Object arg)
	{
		// If we change preset views:
		if (evt.target.equals(Predef_box))
		{
			switchViews((String) arg);
		} // if
		// If we change palettes:
		else if (evt.target.equals(Pal_box))
		{
			String val= new String((String) arg);
			if (val.compareTo("Rainbow") == 0)
				rp_b= true;
			else
				rp_b= false;
		} // else if
		// If we click the Render button:
		else if (evt.target.equals(Render_btn))
		{
			try
			{
				// Verify user inputs:
				// Make sure the inputs are doubles:

				Double dbl_temp= new Double(tf_xmax.getText());
				xmax= dbl_temp.doubleValue();
				dbl_temp= new Double(tf_xmin.getText());
				xmin= dbl_temp.doubleValue();
				dbl_temp= new Double(tf_ymin.getText());
				ymin= dbl_temp.doubleValue();
				dbl_temp= new Double(tf_ymax.getText());
				ymax= dbl_temp.doubleValue();
				// Make sure they fit the constraints:
				if (xmax < -2.25 || xmax > 0.75)
				{
					mandelCanvas1.drawError("ERROR: XMax must be between -2.25 and 0.75.");
				} 
				else if (xmin < -2.25 || xmin > 0.75)
				{
					mandelCanvas1.drawError("ERROR: XMin must be between -2.25 and 0.75.");
				}
				else if (ymax < -1.5 || ymax > 1.5)
				{
					mandelCanvas1.drawError("ERROR: YMax must be between -1.5 and 1.5.");
				} 
				else if (ymin < -1.5 || ymin > 1.5)
				{
					mandelCanvas1.drawError("ERROR: YMin must be between -1.5 and 1.5.");
				} 
				// If all else is okay, they lets do the picture:
				else
				{
					mandelCanvas1.drawError("Rendering image... please wait...");
					mandelCanvas1.drawMandel(xmin, xmax, ymin, ymax, rp_b);
				} 
			}
			catch (NumberFormatException e)
			{
				mandelCanvas1.drawError("ERROR: Inputs are not numbers.");
			}
		} // else if
		// Let the Applet class handle everything else:
		else
		{
			return super.action(evt, arg);
		}
		return true;
	}

	// switchViews() changes the preset view and adjusts the X and Y
	// mins and maxes to the preset values.  The user can still enter
	// their own values if the want to.
	private void switchViews(String new_view)
	{
		Preset p= presets.findPreset(new_view);

		if (p == null)
			return;

		tf_xmin.setText("" + p.xmin);
		tf_xmax.setText("" + p.xmax);
		tf_ymin.setText("" + p.ymin);
		tf_ymax.setText("" + p.ymax);
		mandelCanvas1.setIterations(p.iterations);

	} // switchViews()

	public void zoom(int px1, int px2, int py1, int py2, int w, int h)
	{
		double dx= (xmax - xmin) /  w;
		double dy= (ymax - ymin) /  h;

		xmax= xmin + dx * px2;
		xmin= xmin + dx * px1;

		ymax= ymin + dy * py1;
		ymin= ymin + dy * py2;

		mandelCanvas1.drawMandel(xmin, xmax, ymin, ymax, rp_b);
	}

	public void zoom21(double x1, double y1, double x2, double y2)
	{
		System.out.println("x1=" + x1 + " ,x2=" + x2 + ", y1=" + y1 + ", y2=" + y2);

		double d= xmax - xmin;
		double h= ymax - ymin;

		System.out.println("d=" + d);
		System.out.println("h=" + h);

		xmax= xmin + x2 * d;
		xmin += x1 * d;

		ymax= ymin + y2 * h;
		ymin += y1 * h;
		System.out.println("xmin=" + xmin + " ,xmax=" + xmax + ", ymin=" + ymin + ", ymax=" + ymax);
		mandelCanvas1.drawMandel(xmin, xmax, ymin, ymax, rp_b);
	}

	// Including a main() function (and the MandelFrame class)
	// lets us run MM JAVA! as both an applet and an application.
	public static void main(String[] args)
	{
		new MandelFrame(new MandelMain());
	}

	// about() prints the "about" information to the canvas:
	public void about()
	{
		mandelCanvas1.about();
	}

}